"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));

// src/index.ts
var import_globals = __toESM(require("globals"));

// src/utils/ast.ts
function getStringValue(node) {
  if (!node)
    return "";
  return node.type === "Identifier" ? node.name : node.type === "TemplateLiteral" ? node.quasis[0].value.raw : node.type === "Literal" && typeof node.value === "string" ? node.value : "";
}
function getRawValue(node) {
  return node.type === "Literal" ? node.raw : void 0;
}
function isIdentifier(node, name) {
  return node.type === "Identifier" && (typeof name === "string" ? node.name === name : name.test(node.name));
}
function isLiteral(node, type, value) {
  return node.type === "Literal" && (value === void 0 ? typeof node.value === type : node.value === value);
}
var isTemplateLiteral = (node, value) => node.type === "TemplateLiteral" && node.quasis.length === 1 && // bail out if not simple
(value === void 0 || node.quasis[0].value.raw === value);
function isStringLiteral(node, value) {
  return isLiteral(node, "string", value);
}
function isBooleanLiteral(node, value) {
  return isLiteral(node, "boolean", value);
}
function isStringNode(node) {
  return node && (isStringLiteral(node) || isTemplateLiteral(node));
}
function isPropertyAccessor(node, name) {
  return getStringValue(node.property) === name;
}
function getTestNames(context) {
  const aliases = context.settings.playwright?.globalAliases?.test ?? [];
  return ["test", ...aliases];
}
function isTestIdentifier(context, node) {
  const testNames = getTestNames(context);
  const regex = new RegExp(`^(${testNames.join("|")})$`);
  return isIdentifier(node, regex) || node.type === "MemberExpression" && isIdentifier(node.object, regex);
}
var describeProperties = /* @__PURE__ */ new Set([
  "parallel",
  "serial",
  "only",
  "skip",
  "fixme"
]);
function isDescribeCall(node) {
  const inner = node.type === "CallExpression" ? node.callee : node;
  if (isIdentifier(inner, "describe")) {
    return true;
  }
  if (inner.type !== "MemberExpression") {
    return false;
  }
  return isPropertyAccessor(inner, "describe") ? true : describeProperties.has(getStringValue(inner.property)) ? isDescribeCall(inner.object) : false;
}
function getParent(node) {
  return node.parent;
}
function findParent(node, type) {
  if (!node.parent)
    return;
  return node.parent.type === type ? node.parent : findParent(node.parent, type);
}
function isTestCall(context, node, modifiers) {
  return isTestIdentifier(context, node.callee) && !isDescribeCall(node) && (node.callee.type !== "MemberExpression" || !modifiers || modifiers?.includes(getStringValue(node.callee.property))) && node.arguments.length === 2 && isFunction(node.arguments[1]);
}
var testHooks = /* @__PURE__ */ new Set(["afterAll", "afterEach", "beforeAll", "beforeEach"]);
function isTestHook(context, node) {
  return node.callee.type === "MemberExpression" && isTestIdentifier(context, node.callee.object) && testHooks.has(getStringValue(node.callee.property));
}
function parseFnCall(context, node) {
  if (isTestCall(context, node)) {
    return {
      fn: node.arguments[1],
      name: getStringValue(node.callee),
      type: "test"
    };
  }
  if (node.callee.type === "MemberExpression" && isTestIdentifier(context, node.callee.object) && testHooks.has(getStringValue(node.callee.property))) {
    return {
      fn: node.arguments[0],
      name: getStringValue(node.callee.property),
      type: "hook"
    };
  }
  if (isDescribeCall(node)) {
    return {
      name: getStringValue(node.callee),
      type: "describe"
    };
  }
}
var expectSubCommands = /* @__PURE__ */ new Set(["soft", "poll"]);
function getExpectType(context, node) {
  const aliases = context.settings.playwright?.globalAliases?.expect ?? [];
  const expectNames = ["expect", ...aliases];
  const regex = new RegExp(`(^(${expectNames.join("|")})|Expect)$`);
  if (isIdentifier(node.callee, regex)) {
    return "standalone";
  }
  if (node.callee.type === "MemberExpression" && // TODO: Maybe
  isIdentifier(node.callee.object, "expect")) {
    const type = getStringValue(node.callee.property);
    return expectSubCommands.has(type) ? type : void 0;
  }
}
function isExpectCall(context, node) {
  return !!getExpectType(context, node);
}
function getMatchers(node, chain = []) {
  if (node.parent.type === "MemberExpression" && node.parent.object === node) {
    return getMatchers(node.parent, [
      ...chain,
      node.parent.property
    ]);
  }
  return chain;
}
function dig(node, identifier) {
  return node.type === "MemberExpression" ? dig(node.property, identifier) : node.type === "CallExpression" ? dig(node.callee, identifier) : node.type === "Identifier" ? isIdentifier(node, identifier) : false;
}
function isPageMethod(node, name) {
  return node.callee.type === "MemberExpression" && dig(node.callee.object, /(^(page|frame)|(Page|Frame)$)/) && isPropertyAccessor(node.callee, name);
}
function isFunction(node) {
  return node.type === "ArrowFunctionExpression" || node.type === "FunctionExpression";
}
var equalityMatchers = /* @__PURE__ */ new Set(["toBe", "toEqual", "toStrictEqual"]);

// src/rules/expect-expect.ts
function isAssertionCall(context, node, assertFunctionNames) {
  return isExpectCall(context, node) || assertFunctionNames.find((name) => dig(node.callee, name));
}
var expect_expect_default = {
  create(context) {
    const options = {
      assertFunctionNames: [],
      ...context.options?.[0] ?? {}
    };
    const unchecked = [];
    function checkExpressions(nodes) {
      for (const node of nodes) {
        const index2 = node.type === "CallExpression" ? unchecked.indexOf(node) : -1;
        if (index2 !== -1) {
          unchecked.splice(index2, 1);
          break;
        }
      }
    }
    return {
      CallExpression(node) {
        if (isTestCall(context, node, ["fixme", "only", "skip"])) {
          unchecked.push(node);
        } else if (isAssertionCall(context, node, options.assertFunctionNames)) {
          const ancestors = context.sourceCode.getAncestors(node);
          checkExpressions(ancestors);
        }
      },
      "Program:exit"() {
        unchecked.forEach((node) => {
          context.report({ messageId: "noAssertions", node });
        });
      }
    };
  },
  meta: {
    docs: {
      category: "Best Practices",
      description: "Enforce assertion to be made in a test body",
      recommended: true,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/expect-expect.md"
    },
    messages: {
      noAssertions: "Test has no assertions"
    },
    schema: [
      {
        additionalProperties: false,
        properties: {
          assertFunctionNames: {
            items: [{ type: "string" }],
            type: "array"
          }
        },
        type: "object"
      }
    ],
    type: "problem"
  }
};

// src/rules/max-expects.ts
var max_expects_default = {
  create(context) {
    const options = {
      max: 5,
      ...context.options?.[0] ?? {}
    };
    let count = 0;
    const maybeResetCount = (node) => {
      const parent = getParent(node);
      const isTestFn = parent?.type !== "CallExpression" || isTestCall(context, parent);
      if (isTestFn) {
        count = 0;
      }
    };
    return {
      ArrowFunctionExpression: maybeResetCount,
      "ArrowFunctionExpression:exit": maybeResetCount,
      CallExpression(node) {
        if (!getExpectType(context, node))
          return;
        count += 1;
        if (count > options.max) {
          context.report({
            data: {
              count: count.toString(),
              max: options.max.toString()
            },
            messageId: "exceededMaxAssertion",
            node
          });
        }
      },
      FunctionExpression: maybeResetCount,
      "FunctionExpression:exit": maybeResetCount
    };
  },
  meta: {
    docs: {
      category: "Best Practices",
      description: "Enforces a maximum number assertion calls in a test body",
      recommended: false,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/max-expects.md"
    },
    messages: {
      exceededMaxAssertion: "Too many assertion calls ({{ count }}) - maximum allowed is {{ max }}"
    },
    schema: [
      {
        additionalProperties: false,
        properties: {
          max: {
            minimum: 1,
            type: "integer"
          }
        },
        type: "object"
      }
    ],
    type: "suggestion"
  }
};

// src/rules/max-nested-describe.ts
var max_nested_describe_default = {
  create(context) {
    const { options } = context;
    const max = options[0]?.max ?? 5;
    const describeCallbackStack = [];
    function pushDescribeCallback(node) {
      if (node.parent.type !== "CallExpression" || !isDescribeCall(node.parent)) {
        return;
      }
      describeCallbackStack.push(0);
      if (describeCallbackStack.length > max) {
        context.report({
          data: {
            depth: describeCallbackStack.length.toString(),
            max: max.toString()
          },
          messageId: "exceededMaxDepth",
          node: node.parent.callee
        });
      }
    }
    function popDescribeCallback(node) {
      const { parent } = node;
      if (parent.type === "CallExpression" && isDescribeCall(parent)) {
        describeCallbackStack.pop();
      }
    }
    return {
      ArrowFunctionExpression: pushDescribeCallback,
      "ArrowFunctionExpression:exit": popDescribeCallback,
      FunctionExpression: pushDescribeCallback,
      "FunctionExpression:exit": popDescribeCallback
    };
  },
  meta: {
    docs: {
      category: "Best Practices",
      description: "Enforces a maximum depth to nested describe calls",
      recommended: true,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/max-nested-describe.md"
    },
    messages: {
      exceededMaxDepth: "Maximum describe call depth exceeded ({{ depth }}). Maximum allowed is {{ max }}."
    },
    schema: [
      {
        additionalProperties: false,
        properties: {
          max: {
            minimum: 0,
            type: "integer"
          }
        },
        type: "object"
      }
    ],
    type: "suggestion"
  }
};

// src/rules/missing-playwright-await.ts
var validTypes = /* @__PURE__ */ new Set([
  "AwaitExpression",
  "ReturnStatement",
  "ArrowFunctionExpression"
]);
var expectPlaywrightMatchers = [
  "toBeChecked",
  "toBeDisabled",
  "toBeEnabled",
  "toEqualText",
  // deprecated
  "toEqualUrl",
  "toEqualValue",
  "toHaveFocus",
  "toHaveSelector",
  "toHaveSelectorCount",
  "toHaveText",
  // deprecated
  "toMatchAttribute",
  "toMatchComputedStyle",
  "toMatchText",
  "toMatchTitle",
  "toMatchURL",
  "toMatchValue",
  "toPass"
];
var playwrightTestMatchers = [
  "toBeChecked",
  "toBeDisabled",
  "toBeEditable",
  "toBeEmpty",
  "toBeEnabled",
  "toBeFocused",
  "toBeHidden",
  "toBeVisible",
  "toContainText",
  "toHaveAttribute",
  "toHaveClass",
  "toHaveCount",
  "toHaveCSS",
  "toHaveId",
  "toHaveJSProperty",
  "toBeOK",
  "toHaveScreenshot",
  "toHaveText",
  "toHaveTitle",
  "toHaveURL",
  "toHaveValue",
  "toHaveValues",
  "toBeAttached",
  "toBeInViewport"
];
function getCallType(context, node, awaitableMatchers) {
  if (node.callee.type === "MemberExpression" && isIdentifier(node.callee.object, "test") && isPropertyAccessor(node.callee, "step")) {
    return { messageId: "testStep", node };
  }
  const expectType = getExpectType(context, node);
  if (!expectType)
    return;
  const [lastMatcher] = getMatchers(node).slice(-1);
  const grandparent = lastMatcher?.parent?.parent;
  if (grandparent?.type !== "CallExpression")
    return;
  const matcherName = getStringValue(lastMatcher);
  if (expectType === "poll" || awaitableMatchers.has(matcherName)) {
    return {
      data: { matcherName },
      messageId: expectType === "poll" ? "expectPoll" : "expect",
      node: grandparent
    };
  }
}
var missing_playwright_await_default = {
  create(context) {
    const options = context.options[0] || {};
    const awaitableMatchers = /* @__PURE__ */ new Set([
      ...expectPlaywrightMatchers,
      ...playwrightTestMatchers,
      // Add any custom matchers to the set
      ...options.customMatchers || []
    ]);
    function checkValidity(node) {
      if (validTypes.has(node.parent.type))
        return true;
      if (node.parent.type === "ArrayExpression") {
        return checkValidity(node.parent);
      }
      if (node.parent.type === "CallExpression" && node.parent.callee.type === "MemberExpression" && isIdentifier(node.parent.callee.object, "Promise") && isIdentifier(node.parent.callee.property, "all")) {
        return true;
      }
      if (node.parent.type === "VariableDeclarator") {
        const scope = context.sourceCode.getScope(node.parent.parent);
        for (const ref of scope.references) {
          const refParent = ref.identifier.parent;
          if (validTypes.has(refParent.type))
            return true;
          if (checkValidity(refParent))
            return true;
        }
      }
      return false;
    }
    return {
      CallExpression(node) {
        const result = getCallType(context, node, awaitableMatchers);
        const isValid = result ? checkValidity(result.node) : false;
        if (result && !isValid) {
          context.report({
            data: result.data,
            fix: (fixer) => fixer.insertTextBefore(node, "await "),
            messageId: result.messageId,
            node: node.callee
          });
        }
      }
    };
  },
  meta: {
    docs: {
      category: "Possible Errors",
      description: `Identify false positives when async Playwright APIs are not properly awaited.`,
      recommended: true
    },
    fixable: "code",
    messages: {
      expect: "'{{matcherName}}' must be awaited or returned.",
      expectPoll: "'expect.poll' matchers must be awaited or returned.",
      testStep: "'test.step' must be awaited or returned."
    },
    schema: [
      {
        additionalProperties: false,
        properties: {
          customMatchers: {
            items: { type: "string" },
            type: "array"
          }
        },
        type: "object"
      }
    ],
    type: "problem"
  }
};

// src/rules/no-commented-out-tests.ts
function hasTests(context, node) {
  const testNames = getTestNames(context);
  const names = testNames.join("|");
  const regex = new RegExp(
    `^\\s*(${names}|describe)(\\.\\w+|\\[['"]\\w+['"]\\])?\\s*\\(`,
    "mu"
  );
  return regex.test(node.value);
}
var no_commented_out_tests_default = {
  create(context) {
    function checkNode(node) {
      if (!hasTests(context, node))
        return;
      context.report({
        messageId: "commentedTests",
        node
      });
    }
    return {
      Program() {
        context.sourceCode.getAllComments().forEach(checkNode);
      }
    };
  },
  meta: {
    docs: {
      category: "Best Practices",
      description: "Disallow commented out tests",
      recommended: true,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/no-commented-out-tests.md"
    },
    messages: {
      commentedTests: "Some tests seem to be commented"
    },
    type: "problem"
  }
};

// src/rules/no-conditional-expect.ts
var isCatchCall = (node) => node.callee.type === "MemberExpression" && isPropertyAccessor(node.callee, "catch");
var getTestCallExpressionsFromDeclaredVariables = (context, declaredVariables) => {
  return declaredVariables.reduce(
    (acc, { references }) => [
      ...acc,
      ...references.map(({ identifier }) => getParent(identifier)).filter(
        (node) => node?.type === "CallExpression" && isTestCall(context, node)
      )
    ],
    []
  );
};
var no_conditional_expect_default = {
  create(context) {
    let conditionalDepth = 0;
    let inTestCase = false;
    let inPromiseCatch = false;
    const increaseConditionalDepth = () => inTestCase && conditionalDepth++;
    const decreaseConditionalDepth = () => inTestCase && conditionalDepth--;
    return {
      CallExpression(node) {
        if (isTestCall(context, node)) {
          inTestCase = true;
        }
        if (isCatchCall(node)) {
          inPromiseCatch = true;
        }
        const expectType = getExpectType(context, node);
        if (inTestCase && expectType && conditionalDepth > 0) {
          context.report({
            messageId: "conditionalExpect",
            node
          });
        }
        if (inPromiseCatch && expectType) {
          context.report({
            messageId: "conditionalExpect",
            node
          });
        }
      },
      "CallExpression:exit"(node) {
        if (isTestCall(context, node)) {
          inTestCase = false;
        }
        if (isCatchCall(node)) {
          inPromiseCatch = false;
        }
      },
      CatchClause: increaseConditionalDepth,
      "CatchClause:exit": decreaseConditionalDepth,
      ConditionalExpression: increaseConditionalDepth,
      "ConditionalExpression:exit": decreaseConditionalDepth,
      FunctionDeclaration(node) {
        const declaredVariables = context.sourceCode.getDeclaredVariables(node);
        const testCallExpressions = getTestCallExpressionsFromDeclaredVariables(
          context,
          declaredVariables
        );
        if (testCallExpressions.length > 0) {
          inTestCase = true;
        }
      },
      IfStatement: increaseConditionalDepth,
      "IfStatement:exit": decreaseConditionalDepth,
      LogicalExpression: increaseConditionalDepth,
      "LogicalExpression:exit": decreaseConditionalDepth,
      SwitchStatement: increaseConditionalDepth,
      "SwitchStatement:exit": decreaseConditionalDepth
    };
  },
  meta: {
    docs: {
      category: "Best Practices",
      description: "Disallow calling `expect` conditionally",
      recommended: true,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/no-conditional-expect.md"
    },
    messages: {
      conditionalExpect: "Avoid calling `expect` conditionally`"
    },
    type: "problem"
  }
};

// src/rules/no-conditional-in-test.ts
var no_conditional_in_test_default = {
  create(context) {
    function checkConditional(node) {
      const call = findParent(node, "CallExpression");
      if (call && isTestCall(context, call)) {
        context.report({ messageId: "conditionalInTest", node });
      }
    }
    return {
      ConditionalExpression: checkConditional,
      IfStatement: checkConditional,
      LogicalExpression: checkConditional,
      SwitchStatement: checkConditional
    };
  },
  meta: {
    docs: {
      category: "Best Practices",
      description: "Disallow conditional logic in tests",
      recommended: true,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/no-conditional-in-test.md"
    },
    messages: {
      conditionalInTest: "Avoid having conditionals in tests"
    },
    schema: [],
    type: "problem"
  }
};

// src/rules/no-duplicate-hooks.ts
var no_duplicate_hooks_default = {
  create(context) {
    const hookContexts = [{}];
    return {
      CallExpression(node) {
        if (isDescribeCall(node)) {
          hookContexts.push({});
        }
        if (!isTestHook(context, node)) {
          return;
        }
        const currentLayer = hookContexts[hookContexts.length - 1];
        const name = node.callee.type === "MemberExpression" ? getStringValue(node.callee.property) : "";
        currentLayer[name] || (currentLayer[name] = 0);
        currentLayer[name] += 1;
        if (currentLayer[name] > 1) {
          context.report({
            data: { hook: name },
            messageId: "noDuplicateHook",
            node
          });
        }
      },
      "CallExpression:exit"(node) {
        if (isDescribeCall(node)) {
          hookContexts.pop();
        }
      }
    };
  },
  meta: {
    docs: {
      category: "Best Practices",
      description: "Disallow duplicate setup and teardown hooks",
      recommended: false,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/no-duplicate-hooks.md"
    },
    messages: {
      noDuplicateHook: "Duplicate {{ hook }} in describe block"
    },
    type: "suggestion"
  }
};

// src/rules/no-element-handle.ts
function getPropertyRange(node) {
  return node.type === "Identifier" ? node.range : [node.range[0] + 1, node.range[1] - 1];
}
var no_element_handle_default = {
  create(context) {
    return {
      CallExpression(node) {
        if (isPageMethod(node, "$") || isPageMethod(node, "$$")) {
          context.report({
            messageId: "noElementHandle",
            node: node.callee,
            suggest: [
              {
                fix: (fixer) => {
                  const { property } = node.callee;
                  const fixes = [
                    fixer.replaceTextRange(
                      getPropertyRange(property),
                      "locator"
                    )
                  ];
                  if (node.parent.type === "AwaitExpression") {
                    fixes.push(
                      fixer.removeRange([
                        node.parent.range[0],
                        node.range[0]
                      ])
                    );
                  }
                  return fixes;
                },
                messageId: isPageMethod(node, "$") ? "replaceElementHandleWithLocator" : "replaceElementHandlesWithLocator"
              }
            ]
          });
        }
      }
    };
  },
  meta: {
    docs: {
      category: "Possible Errors",
      description: "The use of ElementHandle is discouraged, use Locator instead",
      recommended: true,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/no-element-handle.md"
    },
    hasSuggestions: true,
    messages: {
      noElementHandle: "Unexpected use of element handles.",
      replaceElementHandlesWithLocator: "Replace `page.$$` with `page.locator`",
      replaceElementHandleWithLocator: "Replace `page.$` with `page.locator`"
    },
    type: "suggestion"
  }
};

// src/rules/no-eval.ts
var no_eval_default = {
  create(context) {
    return {
      CallExpression(node) {
        const isEval = isPageMethod(node, "$eval");
        if (isEval || isPageMethod(node, "$$eval")) {
          context.report({
            messageId: isEval ? "noEval" : "noEvalAll",
            node: node.callee
          });
        }
      }
    };
  },
  meta: {
    docs: {
      category: "Possible Errors",
      description: "The use of `page.$eval` and `page.$$eval` are discouraged, use `locator.evaluate` or `locator.evaluateAll` instead",
      recommended: true,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/no-eval.md"
    },
    messages: {
      noEval: "Unexpected use of page.$eval().",
      noEvalAll: "Unexpected use of page.$$eval()."
    },
    type: "problem"
  }
};

// src/rules/no-focused-test.ts
var no_focused_test_default = {
  create(context) {
    return {
      CallExpression(node) {
        if ((isTestCall(context, node) || isDescribeCall(node)) && node.callee.type === "MemberExpression" && isPropertyAccessor(node.callee, "only")) {
          const { callee } = node;
          context.report({
            messageId: "noFocusedTest",
            node: node.callee.property,
            suggest: [
              {
                // - 1 to remove the `.only` annotation with dot notation
                fix: (fixer) => fixer.removeRange([
                  callee.property.range[0] - 1,
                  callee.range[1]
                ]),
                messageId: "suggestRemoveOnly"
              }
            ]
          });
        }
      }
    };
  },
  meta: {
    docs: {
      category: "Possible Errors",
      description: "Prevent usage of `.only()` focus test annotation",
      recommended: true,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/no-focused-test.md"
    },
    hasSuggestions: true,
    messages: {
      noFocusedTest: "Unexpected focused test.",
      suggestRemoveOnly: "Remove .only() annotation."
    },
    type: "problem"
  }
};

// src/rules/no-force-option.ts
function isForceOptionEnabled(node) {
  const arg = node.arguments.at(-1);
  return arg?.type === "ObjectExpression" && arg.properties.find(
    (property) => property.type === "Property" && getStringValue(property.key) === "force" && isBooleanLiteral(property.value, true)
  );
}
var methodsWithForceOption = /* @__PURE__ */ new Set([
  "check",
  "uncheck",
  "click",
  "dblclick",
  "dragTo",
  "fill",
  "hover",
  "selectOption",
  "selectText",
  "setChecked",
  "tap"
]);
var no_force_option_default = {
  create(context) {
    return {
      MemberExpression(node) {
        if (methodsWithForceOption.has(getStringValue(node.property)) && node.parent.type === "CallExpression") {
          const reportNode = isForceOptionEnabled(node.parent);
          if (reportNode) {
            context.report({ messageId: "noForceOption", node: reportNode });
          }
        }
      }
    };
  },
  meta: {
    docs: {
      category: "Best Practices",
      description: "Prevent usage of `{ force: true }` option.",
      recommended: true,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/no-force-option.md"
    },
    messages: {
      noForceOption: "Unexpected use of { force: true } option."
    },
    type: "suggestion"
  }
};

// src/rules/no-get-by-title.ts
var no_get_by_title_default = {
  create(context) {
    return {
      CallExpression(node) {
        if (isPageMethod(node, "getByTitle")) {
          context.report({ messageId: "noGetByTitle", node });
        }
      }
    };
  },
  meta: {
    docs: {
      category: "Best Practices",
      description: "Disallows the usage of getByTitle()",
      recommended: false,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/no-get-by-title.md"
    },
    messages: {
      noGetByTitle: "The HTML title attribute is not an accessible name. Prefer getByRole() or getByLabelText() instead."
    },
    type: "suggestion"
  }
};

// src/rules/no-hooks.ts
var no_hooks_default = {
  create(context) {
    const options = {
      allow: [],
      ...context.options?.[0] ?? {}
    };
    return {
      CallExpression(node) {
        const call = parseFnCall(context, node);
        if (call?.type === "hook" && !options.allow.includes(call.name)) {
          context.report({
            data: { hookName: call.name },
            messageId: "unexpectedHook",
            node
          });
        }
      }
    };
  },
  meta: {
    docs: {
      category: "Best Practices",
      description: "Disallow setup and teardown hooks",
      recommended: false,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/no-hooks.md"
    },
    messages: {
      unexpectedHook: "Unexpected '{{ hookName }}' hook"
    },
    schema: [
      {
        additionalProperties: false,
        properties: {
          allow: {
            contains: ["beforeAll", "beforeEach", "afterAll", "afterEach"],
            type: "array"
          }
        },
        type: "object"
      }
    ],
    type: "suggestion"
  }
};

// src/rules/no-nested-step.ts
function isStepCall(node) {
  const inner = node.type === "CallExpression" ? node.callee : node;
  if (inner.type !== "MemberExpression") {
    return false;
  }
  return isPropertyAccessor(inner, "step");
}
var no_nested_step_default = {
  create(context) {
    const stack = [];
    function pushStepCallback(node) {
      if (node.parent.type !== "CallExpression" || !isStepCall(node.parent)) {
        return;
      }
      stack.push(0);
      if (stack.length > 1) {
        context.report({
          messageId: "noNestedStep",
          node: node.parent.callee
        });
      }
    }
    function popStepCallback(node) {
      const { parent } = node;
      if (parent.type === "CallExpression" && isStepCall(parent)) {
        stack.pop();
      }
    }
    return {
      ArrowFunctionExpression: pushStepCallback,
      "ArrowFunctionExpression:exit": popStepCallback,
      FunctionExpression: pushStepCallback,
      "FunctionExpression:exit": popStepCallback
    };
  },
  meta: {
    docs: {
      category: "Best Practices",
      description: "Disallow nested `test.step()` methods",
      recommended: true,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/no-nested-step.md"
    },
    messages: {
      noNestedStep: "Do not nest `test.step()` methods."
    },
    schema: [],
    type: "problem"
  }
};

// src/rules/no-networkidle.ts
var messageId = "noNetworkidle";
var methods = /* @__PURE__ */ new Set([
  "goBack",
  "goForward",
  "goto",
  "reload",
  "setContent",
  "waitForLoadState",
  "waitForURL"
]);
var no_networkidle_default = {
  create(context) {
    return {
      CallExpression(node) {
        if (node.callee.type !== "MemberExpression")
          return;
        const methodName = getStringValue(node.callee.property);
        if (!methods.has(methodName))
          return;
        if (methodName === "waitForLoadState") {
          const arg = node.arguments[0];
          if (arg && isStringLiteral(arg, "networkidle")) {
            context.report({ messageId, node: arg });
          }
          return;
        }
        if (node.arguments.length >= 2) {
          const [_, arg] = node.arguments;
          if (arg.type !== "ObjectExpression")
            return;
          const property = arg.properties.filter((p) => p.type === "Property").find((p) => isStringLiteral(p.value, "networkidle"));
          if (property) {
            context.report({ messageId, node: property.value });
          }
        }
      }
    };
  },
  meta: {
    docs: {
      category: "Possible Errors",
      description: "Prevent usage of the networkidle option",
      recommended: true
    },
    messages: {
      noNetworkidle: "Unexpected use of networkidle."
    },
    type: "problem"
  }
};

// src/rules/no-nth-methods.ts
var methods2 = /* @__PURE__ */ new Set(["first", "last", "nth"]);
var no_nth_methods_default = {
  create(context) {
    return {
      CallExpression(node) {
        if (node.callee.type !== "MemberExpression")
          return;
        const method = getStringValue(node.callee.property);
        if (!methods2.has(method))
          return;
        context.report({
          data: { method },
          loc: {
            end: node.loc.end,
            start: node.callee.property.loc.start
          },
          messageId: "noNthMethod"
        });
      }
    };
  },
  meta: {
    docs: {
      category: "Best Practices",
      description: "Disallow usage of nth methods",
      recommended: true,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/no-nth-methods.md"
    },
    messages: {
      noNthMethod: "Unexpected use of {{method}}()"
    },
    type: "problem"
  }
};

// src/rules/no-page-pause.ts
var no_page_pause_default = {
  create(context) {
    return {
      CallExpression(node) {
        if (isPageMethod(node, "pause")) {
          context.report({ messageId: "noPagePause", node });
        }
      }
    };
  },
  meta: {
    docs: {
      category: "Possible Errors",
      description: "Prevent usage of page.pause()",
      recommended: true
    },
    messages: {
      noPagePause: "Unexpected use of page.pause()."
    },
    type: "problem"
  }
};

// src/rules/no-raw-locators.ts
function normalize(str) {
  const match = /\[([^=]+?)=['"]?([^'"]+?)['"]?\]/.exec(str);
  return match ? `[${match[1]}=${match[2]}]` : str;
}
var no_raw_locators_default = {
  create(context) {
    const options = {
      allowed: [],
      ...context.options?.[0] ?? {}
    };
    function isAllowed(arg) {
      return options.allowed.some((a) => normalize(a) === normalize(arg));
    }
    return {
      CallExpression(node) {
        if (node.callee.type !== "MemberExpression")
          return;
        const method = getStringValue(node.callee.property);
        const arg = getStringValue(node.arguments[0]);
        const isLocator = isPageMethod(node, "locator") || method === "locator";
        if (isLocator && !isAllowed(arg)) {
          context.report({ messageId: "noRawLocator", node });
        }
      }
    };
  },
  meta: {
    docs: {
      category: "Best Practices",
      description: "Disallows the usage of raw locators",
      recommended: false,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/no-raw-locators.md"
    },
    messages: {
      noRawLocator: "Usage of raw locator detected. Use methods like .getByRole() or .getByText() instead of raw locators."
    },
    schema: [
      {
        additionalProperties: false,
        properties: {
          allowed: {
            items: { type: "string" },
            type: "array"
          }
        },
        type: "object"
      }
    ],
    type: "suggestion"
  }
};

// src/utils/parseExpectCall.ts
var MODIFIER_NAMES = /* @__PURE__ */ new Set(["not", "resolves", "rejects"]);
function getExpectArguments(node) {
  const grandparent = node.parent.parent;
  return grandparent.type === "CallExpression" ? grandparent.arguments : [];
}
function parseExpectCall(context, node) {
  if (!isExpectCall(context, node)) {
    return;
  }
  const members = getMatchers(node);
  const modifiers = [];
  let matcher;
  members.forEach((item) => {
    if (MODIFIER_NAMES.has(getStringValue(item))) {
      modifiers.push(item);
    } else {
      matcher = item;
    }
  });
  if (!matcher) {
    return;
  }
  return {
    args: getExpectArguments(matcher),
    matcher,
    matcherName: getStringValue(matcher),
    members,
    modifiers
  };
}

// src/rules/no-restricted-matchers.ts
var no_restricted_matchers_default = {
  create(context) {
    const restrictedChains = context.options?.[0] ?? {};
    return {
      CallExpression(node) {
        const expectCall = parseExpectCall(context, node);
        if (!expectCall)
          return;
        Object.entries(restrictedChains).map(([restriction, message]) => {
          const chain = expectCall.members;
          const restrictionLinks = restriction.split(".").length;
          const startIndex = chain.findIndex((_, i) => {
            const partial = chain.slice(i, i + restrictionLinks).map(getStringValue).join(".");
            return partial === restriction;
          });
          return {
            // If the restriction chain was found, return the portion of the
            // chain that matches the restriction chain.
            chain: startIndex !== -1 ? chain.slice(startIndex, startIndex + restrictionLinks) : [],
            message,
            restriction
          };
        }).filter(({ chain }) => chain.length).forEach(({ chain, message, restriction }) => {
          context.report({
            data: { message: message ?? "", restriction },
            loc: {
              end: chain.at(-1).loc.end,
              start: chain[0].loc.start
            },
            messageId: message ? "restrictedWithMessage" : "restricted"
          });
        });
      }
    };
  },
  meta: {
    docs: {
      category: "Best Practices",
      description: "Disallow specific matchers & modifiers",
      recommended: false,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/no-restricted-matchers.md"
    },
    messages: {
      restricted: "Use of `{{restriction}}` is disallowed",
      restrictedWithMessage: "{{message}}"
    },
    schema: [
      {
        additionalProperties: {
          type: ["string", "null"]
        },
        type: "object"
      }
    ],
    type: "suggestion"
  }
};

// src/rules/no-skipped-test.ts
var no_skipped_test_default = {
  create(context) {
    return {
      CallExpression(node) {
        const options = context.options[0] || {};
        const allowConditional = !!options.allowConditional;
        const { callee } = node;
        if ((isTestIdentifier(context, callee) || isDescribeCall(node)) && callee.type === "MemberExpression" && isPropertyAccessor(callee, "skip")) {
          const isHook = isTestCall(context, node) || isDescribeCall(node);
          if (!isHook && allowConditional && node.arguments.length) {
            return;
          }
          context.report({
            messageId: "noSkippedTest",
            node: isHook ? callee.property : node,
            suggest: [
              {
                fix: (fixer) => {
                  return isHook ? fixer.removeRange([
                    callee.property.range[0] - 1,
                    callee.range[1]
                  ]) : fixer.remove(node.parent);
                },
                messageId: "removeSkippedTestAnnotation"
              }
            ]
          });
        }
      }
    };
  },
  meta: {
    docs: {
      category: "Best Practices",
      description: "Prevent usage of the `.skip()` skip test annotation.",
      recommended: true,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/no-skipped-test.md"
    },
    hasSuggestions: true,
    messages: {
      noSkippedTest: "Unexpected use of the `.skip()` annotation.",
      removeSkippedTestAnnotation: "Remove the `.skip()` annotation."
    },
    schema: [
      {
        additionalProperties: false,
        properties: {
          allowConditional: {
            default: false,
            type: "boolean"
          }
        },
        type: "object"
      }
    ],
    type: "suggestion"
  }
};

// src/rules/no-standalone-expect.ts
var getBlockType = (statement) => {
  const func = getParent(statement);
  if (!func) {
    throw new Error(
      `Unexpected BlockStatement. No parent defined. - please file a github issue at https://github.com/playwright-community/eslint-plugin-playwright`
    );
  }
  if (func.type === "FunctionDeclaration") {
    return "function";
  }
  if (isFunction(func) && func.parent) {
    const expr = func.parent;
    if (expr.type === "VariableDeclarator" || expr.type === "MethodDefinition") {
      return "function";
    }
    if (expr.type === "CallExpression" && isDescribeCall(expr)) {
      return "describe";
    }
  }
  return null;
};
var no_standalone_expect_default = {
  create(context) {
    const callStack = [];
    return {
      ArrowFunctionExpression(node) {
        if (node.parent?.type !== "CallExpression") {
          callStack.push("arrow");
        }
      },
      "ArrowFunctionExpression:exit"() {
        if (callStack[callStack.length - 1] === "arrow") {
          callStack.pop();
        }
      },
      BlockStatement(statement) {
        const blockType = getBlockType(statement);
        if (blockType) {
          callStack.push(blockType);
        }
      },
      "BlockStatement:exit"(statement) {
        if (callStack[callStack.length - 1] === getBlockType(statement)) {
          callStack.pop();
        }
      },
      CallExpression(node) {
        if (getExpectType(context, node)) {
          const parent = callStack.at(-1);
          if (!parent || parent === "describe") {
            const root = findParent(node, "CallExpression");
            context.report({
              messageId: "unexpectedExpect",
              node: root ?? node
            });
          }
          return;
        }
        if (isTestCall(context, node)) {
          callStack.push("test");
        }
        if (node.callee.type === "TaggedTemplateExpression") {
          callStack.push("template");
        }
      },
      "CallExpression:exit"(node) {
        const top = callStack[callStack.length - 1];
        if (top === "test" && isTestCall(context, node) && node.callee.type !== "MemberExpression" || top === "template" && node.callee.type === "TaggedTemplateExpression") {
          callStack.pop();
        }
      }
    };
  },
  meta: {
    docs: {
      category: "Best Practices",
      description: "Disallow using `expect` outside of `test` blocks",
      recommended: false,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/no-standalone-expect.md"
    },
    fixable: "code",
    messages: {
      unexpectedExpect: "Expect must be inside of a test block"
    },
    type: "suggestion"
  }
};

// src/utils/misc.ts
var getAmountData = (amount) => ({
  amount: amount.toString(),
  s: amount === 1 ? "" : "s"
});
var truthy = Boolean;

// src/rules/no-unsafe-references.ts
function collectVariables(scope) {
  if (!scope || scope.type === "global")
    return [];
  return [
    ...collectVariables(scope.upper),
    ...scope.variables.map((ref) => ref.name)
  ];
}
function addArgument(fixer, node, refs) {
  if (!node.arguments.length)
    return;
  if (node.arguments.length === 1) {
    return fixer.insertTextAfter(node.arguments[0], `, [${refs}]`);
  }
  const arg = node.arguments.at(-1);
  if (!arg)
    return;
  if (arg.type !== "ArrayExpression") {
    return fixer.replaceText(arg, `[${getStringValue(arg)}, ${refs}]`);
  }
  const lastItem = arg.elements.at(-1);
  return lastItem ? fixer.insertTextAfter(lastItem, `, ${refs}`) : fixer.replaceText(arg, `[${refs}]`);
}
function getParen(context, node) {
  let token = context.sourceCode.getFirstToken(node);
  while (token && token.value !== "(") {
    token = context.sourceCode.getTokenAfter(token);
  }
  return token;
}
function addParam(context, fixer, node, refs) {
  const lastParam = node.params.at(-1);
  if (lastParam) {
    return lastParam.type === "ArrayPattern" ? fixer.insertTextAfter(lastParam.elements.at(-1), `, ${refs}`) : fixer.replaceText(lastParam, `[${getStringValue(lastParam)}, ${refs}]`);
  }
  const token = getParen(context, node);
  return token ? fixer.insertTextAfter(token, `[${refs}]`) : null;
}
var no_unsafe_references_default = {
  create(context) {
    return {
      CallExpression(node) {
        if (!isPageMethod(node, "evaluate"))
          return;
        const [fn] = node.arguments;
        if (!fn || !isFunction(fn))
          return;
        const { through, upper } = context.sourceCode.getScope(fn.body);
        const allRefs = new Set(collectVariables(upper));
        through.filter((ref) => {
          const parent = getParent(ref.identifier);
          return parent?.type !== "TSTypeReference";
        }).filter((ref) => allRefs.has(ref.identifier.name)).forEach((ref, i, arr) => {
          const descriptor = {
            data: { variable: ref.identifier.name },
            messageId: "noUnsafeReference",
            node: ref.identifier
          };
          if (i !== 0) {
            context.report(descriptor);
            return;
          }
          context.report({
            ...descriptor,
            fix(fixer) {
              const refs = arr.map((ref2) => ref2.identifier.name).join(", ");
              return [
                addArgument(fixer, node, refs),
                addParam(context, fixer, fn, refs)
              ].filter(truthy);
            }
          });
        });
      }
    };
  },
  meta: {
    docs: {
      category: "Possible Errors",
      description: "Prevent unsafe variable references in page.evaluate()",
      recommended: true,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/no-unsafe-references.md"
    },
    fixable: "code",
    messages: {
      noUnsafeReference: 'Unsafe reference to variable "{{ variable }}" in page.evaluate()'
    },
    type: "problem"
  }
};

// src/rules/no-useless-await.ts
var locatorMethods = /* @__PURE__ */ new Set([
  "and",
  "first",
  "getByAltText",
  "getByLabel",
  "getByPlaceholder",
  "getByRole",
  "getByTestId",
  "getByText",
  "getByTitle",
  "last",
  "locator",
  "nth",
  "or"
]);
var pageMethods = /* @__PURE__ */ new Set([
  "childFrames",
  "frame",
  "frameLocator",
  "frames",
  "isClosed",
  "isDetached",
  "mainFrame",
  "name",
  "on",
  "page",
  "parentFrame",
  "setDefaultNavigationTimeout",
  "setDefaultTimeout",
  "url",
  "video",
  "viewportSize",
  "workers"
]);
function isSupportedMethod(node) {
  if (node.callee.type !== "MemberExpression")
    return false;
  const name = getStringValue(node.callee.property);
  return locatorMethods.has(name) || pageMethods.has(name) && isPageMethod(node, name);
}
var no_useless_await_default = {
  create(context) {
    return {
      AwaitExpression(node) {
        if (node.argument.type !== "CallExpression")
          return;
        const { callee } = node.argument;
        if (callee.type !== "MemberExpression")
          return;
        if (!isSupportedMethod(node.argument))
          return;
        const start = node.loc.start;
        const range = node.range;
        context.report({
          fix: (fixer) => fixer.removeRange([range[0], range[0] + 6]),
          loc: {
            end: {
              column: start.column + 5,
              line: start.line
            },
            start
          },
          messageId: "noUselessAwait"
        });
      }
    };
  },
  meta: {
    docs: {
      category: "Possible Errors",
      description: "Disallow unnecessary awaits for Playwright methods",
      recommended: true
    },
    fixable: "code",
    messages: {
      noUselessAwait: "Unnecessary await expression. This method does not return a Promise."
    },
    type: "problem"
  }
};

// src/utils/fixer.ts
var getRangeOffset = (node) => node.type === "Identifier" ? 0 : 1;
function replaceAccessorFixer(fixer, node, text) {
  const [start, end] = node.range;
  return fixer.replaceTextRange(
    [start + getRangeOffset(node), end - getRangeOffset(node)],
    text
  );
}

// src/rules/no-useless-not.ts
var matcherMap = {
  toBeDisabled: "toBeEnabled",
  toBeEnabled: "toBeDisabled",
  toBeHidden: "toBeVisible",
  toBeVisible: "toBeHidden"
};
var no_useless_not_default = {
  create(context) {
    return {
      CallExpression(node) {
        const expectCall = parseExpectCall(context, node);
        if (!expectCall)
          return;
        const notModifier = expectCall.modifiers.find(
          (mod) => getStringValue(mod) === "not"
        );
        if (!notModifier)
          return;
        if (expectCall.matcherName in matcherMap) {
          const newMatcher = matcherMap[expectCall.matcherName];
          context.report({
            data: { new: newMatcher, old: expectCall.matcherName },
            fix: (fixer) => [
              fixer.removeRange([
                notModifier.range[0] - getRangeOffset(notModifier),
                notModifier.range[1] + 1
              ]),
              replaceAccessorFixer(fixer, expectCall.matcher, newMatcher)
            ],
            loc: {
              end: expectCall.matcher.loc.end,
              start: notModifier.loc.start
            },
            messageId: "noUselessNot"
          });
        }
      }
    };
  },
  meta: {
    docs: {
      category: "Best Practices",
      description: `Disallow usage of 'not' matchers when a more specific matcher exists`,
      recommended: true,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/no-useless-not.md"
    },
    fixable: "code",
    messages: {
      noUselessNot: "Unexpected usage of not.{{old}}(). Use {{new}}() instead."
    },
    type: "problem"
  }
};

// src/rules/no-wait-for-selector.ts
var no_wait_for_selector_default = {
  create(context) {
    return {
      CallExpression(node) {
        if (isPageMethod(node, "waitForSelector")) {
          context.report({
            messageId: "noWaitForSelector",
            node,
            suggest: [
              {
                fix: (fixer) => fixer.remove(
                  node.parent && node.parent.type !== "AwaitExpression" ? node.parent : node.parent.parent
                ),
                messageId: "removeWaitForSelector"
              }
            ]
          });
        }
      }
    };
  },
  meta: {
    docs: {
      category: "Best Practices",
      description: "Prevent usage of page.waitForSelector()",
      recommended: true,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/no-wait-for-selector.md"
    },
    hasSuggestions: true,
    messages: {
      noWaitForSelector: "Unexpected use of page.waitForSelector().",
      removeWaitForSelector: "Remove the page.waitForSelector() method."
    },
    type: "suggestion"
  }
};

// src/rules/no-wait-for-timeout.ts
var no_wait_for_timeout_default = {
  create(context) {
    return {
      CallExpression(node) {
        if (isPageMethod(node, "waitForTimeout")) {
          context.report({
            messageId: "noWaitForTimeout",
            node,
            suggest: [
              {
                fix: (fixer) => fixer.remove(
                  node.parent && node.parent.type !== "AwaitExpression" ? node.parent : node.parent.parent
                ),
                messageId: "removeWaitForTimeout"
              }
            ]
          });
        }
      }
    };
  },
  meta: {
    docs: {
      category: "Best Practices",
      description: "Prevent usage of page.waitForTimeout()",
      recommended: true,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/no-wait-for-timeout.md"
    },
    hasSuggestions: true,
    messages: {
      noWaitForTimeout: "Unexpected use of page.waitForTimeout().",
      removeWaitForTimeout: "Remove the page.waitForTimeout() method."
    },
    type: "suggestion"
  }
};

// src/rules/prefer-comparison-matcher.ts
var isString = (node) => {
  return isStringLiteral(node) || node.type === "TemplateLiteral";
};
var isComparingToString = (expression) => {
  return isString(expression.left) || isString(expression.right);
};
var invertedOperators = {
  "<": ">=",
  "<=": ">",
  ">": "<=",
  ">=": "<"
};
var operatorMatcher = {
  "<": "toBeLessThan",
  "<=": "toBeLessThanOrEqual",
  ">": "toBeGreaterThan",
  ">=": "toBeGreaterThanOrEqual"
};
var determineMatcher = (operator, negated) => {
  const op = negated ? invertedOperators[operator] : operator;
  return operatorMatcher[op] ?? null;
};
var prefer_comparison_matcher_default = {
  create(context) {
    return {
      CallExpression(node) {
        const expectCall = parseExpectCall(context, node);
        if (!expectCall || expectCall.args.length === 0)
          return;
        const { args, matcher } = expectCall;
        const [comparison] = node.arguments;
        const expectCallEnd = node.range[1];
        const [matcherArg] = args;
        if (comparison?.type !== "BinaryExpression" || isComparingToString(comparison) || !equalityMatchers.has(getStringValue(matcher)) || !isBooleanLiteral(matcherArg)) {
          return;
        }
        const hasNot = expectCall.modifiers.some(
          (node2) => getStringValue(node2) === "not"
        );
        const preferredMatcher = determineMatcher(
          comparison.operator,
          getRawValue(matcherArg) === hasNot.toString()
        );
        if (!preferredMatcher) {
          return;
        }
        context.report({
          data: { preferredMatcher },
          fix(fixer) {
            const [modifier] = expectCall.modifiers;
            const modifierText = modifier && getStringValue(modifier) !== "not" ? `.${getStringValue(modifier)}` : "";
            return [
              // Replace the comparison argument with the left-hand side of the comparison
              fixer.replaceText(
                comparison,
                context.sourceCode.getText(comparison.left)
              ),
              // Replace the current matcher & modifier with the preferred matcher
              fixer.replaceTextRange(
                [expectCallEnd, getParent(matcher).range[1]],
                `${modifierText}.${preferredMatcher}`
              ),
              // Replace the matcher argument with the right-hand side of the comparison
              fixer.replaceText(
                matcherArg,
                context.sourceCode.getText(comparison.right)
              )
            ];
          },
          messageId: "useToBeComparison",
          node: matcher
        });
      }
    };
  },
  meta: {
    docs: {
      category: "Best Practices",
      description: "Suggest using the built-in comparison matchers",
      recommended: false
    },
    fixable: "code",
    messages: {
      useToBeComparison: "Prefer using `{{ preferredMatcher }}` instead"
    },
    type: "suggestion"
  }
};

// src/rules/prefer-equality-matcher.ts
var prefer_equality_matcher_default = {
  create(context) {
    return {
      CallExpression(node) {
        const expectCall = parseExpectCall(context, node);
        if (!expectCall || expectCall.args.length === 0)
          return;
        const { args, matcher } = expectCall;
        const [comparison] = node.arguments;
        const expectCallEnd = node.range[1];
        const [matcherArg] = args;
        if (comparison?.type !== "BinaryExpression" || comparison.operator !== "===" && comparison.operator !== "!==" || !equalityMatchers.has(getStringValue(matcher)) || !isBooleanLiteral(matcherArg)) {
          return;
        }
        const matcherValue = getRawValue(matcherArg) === "true";
        const [modifier] = expectCall.modifiers;
        const hasNot = expectCall.modifiers.some(
          (node2) => getStringValue(node2) === "not"
        );
        const addNotModifier = (comparison.operator === "!==" ? !matcherValue : matcherValue) === hasNot;
        context.report({
          messageId: "useEqualityMatcher",
          node: matcher,
          suggest: [...equalityMatchers.keys()].map((equalityMatcher) => ({
            data: { matcher: equalityMatcher },
            fix(fixer) {
              let modifierText = modifier && getStringValue(modifier) !== "not" ? `.${getStringValue(modifier)}` : "";
              if (addNotModifier) {
                modifierText += `.not`;
              }
              return [
                // replace the comparison argument with the left-hand side of the comparison
                fixer.replaceText(
                  comparison,
                  context.sourceCode.getText(comparison.left)
                ),
                // replace the current matcher & modifier with the preferred matcher
                fixer.replaceTextRange(
                  [expectCallEnd, getParent(matcher).range[1]],
                  `${modifierText}.${equalityMatcher}`
                ),
                // replace the matcher argument with the right-hand side of the comparison
                fixer.replaceText(
                  matcherArg,
                  context.sourceCode.getText(comparison.right)
                )
              ];
            },
            messageId: "suggestEqualityMatcher"
          }))
        });
      }
    };
  },
  meta: {
    docs: {
      category: "Best Practices",
      description: "Suggest using the built-in equality matchers",
      recommended: false,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/prefer-equality-matcher.md"
    },
    hasSuggestions: true,
    messages: {
      suggestEqualityMatcher: "Use `{{ matcher }}`",
      useEqualityMatcher: "Prefer using one of the equality matchers instead"
    },
    type: "suggestion"
  }
};

// src/rules/prefer-hooks-in-order.ts
var HooksOrder = ["beforeAll", "beforeEach", "afterEach", "afterAll"];
var prefer_hooks_in_order_default = {
  create(context) {
    let previousHookIndex = -1;
    let inHook = false;
    return {
      CallExpression(node) {
        if (inHook)
          return;
        if (!isTestHook(context, node)) {
          previousHookIndex = -1;
          return;
        }
        inHook = true;
        const currentHook = node.callee.type === "MemberExpression" ? getStringValue(node.callee.property) : "";
        const currentHookIndex = HooksOrder.indexOf(currentHook);
        if (currentHookIndex < previousHookIndex) {
          return context.report({
            data: {
              currentHook,
              previousHook: HooksOrder[previousHookIndex]
            },
            messageId: "reorderHooks",
            node
          });
        }
        previousHookIndex = currentHookIndex;
      },
      "CallExpression:exit"(node) {
        if (isTestHook(context, node)) {
          inHook = false;
          return;
        }
        if (inHook) {
          return;
        }
        previousHookIndex = -1;
      }
    };
  },
  meta: {
    docs: {
      category: "Best Practices",
      description: "Prefer having hooks in a consistent order",
      recommended: false,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/prefer-hooks-in-order.md"
    },
    messages: {
      reorderHooks: "`{{ currentHook }}` hooks should be before any `{{ previousHook }}` hooks"
    },
    type: "suggestion"
  }
};

// src/rules/prefer-hooks-on-top.ts
var prefer_hooks_on_top_default = {
  create(context) {
    const stack = [false];
    return {
      CallExpression(node) {
        if (isTestCall(context, node)) {
          stack[stack.length - 1] = true;
        }
        if (stack.at(-1) && isTestHook(context, node)) {
          context.report({
            messageId: "noHookOnTop",
            node
          });
        }
        stack.push(false);
      },
      "CallExpression:exit"() {
        stack.pop();
      }
    };
  },
  meta: {
    docs: {
      category: "Best Practices",
      description: "Suggest having hooks before any test cases",
      recommended: false,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/prefer-hooks-on-top.md"
    },
    messages: {
      noHookOnTop: "Hooks should come before test cases"
    },
    type: "suggestion"
  }
};

// src/rules/prefer-lowercase-title.ts
var prefer_lowercase_title_default = {
  create(context) {
    const { allowedPrefixes, ignore, ignoreTopLevelDescribe } = {
      allowedPrefixes: [],
      ignore: [],
      ignoreTopLevelDescribe: false,
      ...context.options?.[0] ?? {}
    };
    let describeCount = 0;
    return {
      CallExpression(node) {
        const method = isDescribeCall(node) ? "test.describe" : isTestCall(context, node) ? "test" : null;
        if (method === "test.describe") {
          describeCount++;
          if (ignoreTopLevelDescribe && describeCount === 1) {
            return;
          }
        } else if (!method) {
          return;
        }
        const [title] = node.arguments;
        if (!isStringNode(title)) {
          return;
        }
        const description = getStringValue(title);
        if (!description || allowedPrefixes.some((name) => description.startsWith(name))) {
          return;
        }
        const firstCharacter = description.charAt(0);
        if (!firstCharacter || firstCharacter === firstCharacter.toLowerCase() || ignore.includes(method)) {
          return;
        }
        context.report({
          data: { method },
          fix(fixer) {
            const rangeIgnoringQuotes = [
              title.range[0] + 1,
              title.range[1] - 1
            ];
            const newDescription = description.substring(0, 1).toLowerCase() + description.substring(1);
            return fixer.replaceTextRange(rangeIgnoringQuotes, newDescription);
          },
          messageId: "unexpectedLowercase",
          node: node.arguments[0]
        });
      },
      "CallExpression:exit"(node) {
        if (isDescribeCall(node)) {
          describeCount--;
        }
      }
    };
  },
  meta: {
    docs: {
      category: "Best Practices",
      description: "Enforce lowercase test names",
      recommended: false,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/prefer-lowercase-title.md"
    },
    fixable: "code",
    messages: {
      unexpectedLowercase: "`{{method}}`s should begin with lowercase"
    },
    schema: [
      {
        additionalProperties: false,
        properties: {
          allowedPrefixes: {
            additionalItems: false,
            items: { type: "string" },
            type: "array"
          },
          ignore: {
            additionalItems: false,
            items: {
              enum: ["test.describe", "test"]
            },
            type: "array"
          },
          ignoreTopLevelDescribe: {
            default: false,
            type: "boolean"
          }
        },
        type: "object"
      }
    ],
    type: "suggestion"
  }
};

// src/rules/prefer-strict-equal.ts
var prefer_strict_equal_default = {
  create(context) {
    return {
      CallExpression(node) {
        const expectCall = parseExpectCall(context, node);
        if (expectCall?.matcherName === "toEqual") {
          context.report({
            messageId: "useToStrictEqual",
            node: expectCall.matcher,
            suggest: [
              {
                fix: (fixer) => {
                  return replaceAccessorFixer(
                    fixer,
                    expectCall.matcher,
                    "toStrictEqual"
                  );
                },
                messageId: "suggestReplaceWithStrictEqual"
              }
            ]
          });
        }
      }
    };
  },
  meta: {
    docs: {
      category: "Best Practices",
      description: "Suggest using `toStrictEqual()`",
      recommended: false,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/prefer-strict-equal.md"
    },
    fixable: "code",
    hasSuggestions: true,
    messages: {
      suggestReplaceWithStrictEqual: "Replace with `toStrictEqual()`",
      useToStrictEqual: "Use toStrictEqual() instead"
    },
    schema: [],
    type: "suggestion"
  }
};

// src/rules/prefer-to-be.ts
function shouldUseToBe(expectCall) {
  let arg = expectCall.args[0];
  if (arg.type === "UnaryExpression" && arg.operator === "-") {
    arg = arg.argument;
  }
  if (arg.type === "Literal") {
    return !("regex" in arg);
  }
  return arg.type === "TemplateLiteral";
}
function reportPreferToBe(context, expectCall, whatToBe, notModifier) {
  context.report({
    fix(fixer) {
      const fixes = [
        replaceAccessorFixer(fixer, expectCall.matcher, `toBe${whatToBe}`)
      ];
      if (expectCall.args?.length && whatToBe !== "") {
        fixes.push(fixer.remove(expectCall.args[0]));
      }
      if (notModifier) {
        const [start, end] = notModifier.range;
        fixes.push(fixer.removeRange([start - 1, end]));
      }
      return fixes;
    },
    messageId: `useToBe${whatToBe}`,
    node: expectCall.matcher
  });
}
var prefer_to_be_default = {
  create(context) {
    return {
      CallExpression(node) {
        const expectCall = parseExpectCall(context, node);
        if (!expectCall)
          return;
        const notMatchers = ["toBeUndefined", "toBeDefined"];
        const notModifier = expectCall.modifiers.find(
          (node2) => getStringValue(node2) === "not"
        );
        if (notModifier && notMatchers.includes(expectCall.matcherName)) {
          return reportPreferToBe(
            context,
            expectCall,
            expectCall.matcherName === "toBeDefined" ? "Undefined" : "Defined",
            notModifier
          );
        }
        const firstArg = expectCall.args[0];
        if (!equalityMatchers.has(expectCall.matcherName) || !firstArg) {
          return;
        }
        if (firstArg.type === "Literal" && firstArg.value === null) {
          return reportPreferToBe(context, expectCall, "Null");
        }
        if (isIdentifier(firstArg, "undefined")) {
          const name = notModifier ? "Defined" : "Undefined";
          return reportPreferToBe(context, expectCall, name, notModifier);
        }
        if (isIdentifier(firstArg, "NaN")) {
          return reportPreferToBe(context, expectCall, "NaN");
        }
        if (shouldUseToBe(expectCall) && expectCall.matcherName !== "toBe") {
          reportPreferToBe(context, expectCall, "");
        }
      }
    };
  },
  meta: {
    docs: {
      category: "Best Practices",
      description: "Suggest using `toBe()` for primitive literals",
      recommended: false,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/prefer-to-be.md"
    },
    fixable: "code",
    messages: {
      useToBe: "Use `toBe` when expecting primitive literals",
      useToBeDefined: "Use `toBeDefined` instead",
      useToBeNaN: "Use `toBeNaN` instead",
      useToBeNull: "Use `toBeNull` instead",
      useToBeUndefined: "Use `toBeUndefined` instead"
    },
    schema: [],
    type: "suggestion"
  }
};

// src/rules/prefer-to-contain.ts
var isFixableIncludesCallExpression = (node) => node.type === "CallExpression" && node.callee.type === "MemberExpression" && isPropertyAccessor(node.callee, "includes") && node.arguments.length === 1 && node.arguments[0].type !== "SpreadElement";
var prefer_to_contain_default = {
  create(context) {
    return {
      CallExpression(node) {
        const expectCall = parseExpectCall(context, node);
        if (!expectCall || expectCall.args.length === 0)
          return;
        const { args, matcher, matcherName } = expectCall;
        const [includesCall] = node.arguments;
        const [matcherArg] = args;
        if (!includesCall || matcherArg.type === "SpreadElement" || !equalityMatchers.has(matcherName) || !isBooleanLiteral(matcherArg) || !isFixableIncludesCallExpression(includesCall)) {
          return;
        }
        const notModifier = expectCall.modifiers.find(
          (node2) => getStringValue(node2) === "not"
        );
        context.report({
          fix(fixer) {
            const addNotModifier = matcherArg.type === "Literal" && matcherArg.value === !!notModifier;
            const fixes = [
              // remove the "includes" call entirely
              fixer.removeRange([
                includesCall.callee.property.range[0] - 1,
                includesCall.range[1]
              ]),
              // replace the current matcher with "toContain", adding "not" if needed
              fixer.replaceText(
                matcher,
                addNotModifier ? "not.toContain" : "toContain"
              ),
              // replace the matcher argument with the value from the "includes"
              fixer.replaceText(
                expectCall.args[0],
                context.sourceCode.getText(includesCall.arguments[0])
              )
            ];
            if (notModifier) {
              fixes.push(
                fixer.removeRange([
                  notModifier.range[0],
                  notModifier.range[1] + 1
                ])
              );
            }
            return fixes;
          },
          messageId: "useToContain",
          node: matcher
        });
      }
    };
  },
  meta: {
    docs: {
      category: "Best Practices",
      description: "Suggest using toContain()",
      recommended: false,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/prefer-to-contain.md"
    },
    fixable: "code",
    messages: {
      useToContain: "Use toContain() instead"
    },
    type: "suggestion"
  }
};

// src/rules/prefer-to-have-count.ts
var prefer_to_have_count_default = {
  create(context) {
    return {
      CallExpression(node) {
        const expectCall = parseExpectCall(context, node);
        if (!expectCall || !equalityMatchers.has(expectCall.matcherName)) {
          return;
        }
        const [argument] = node.arguments;
        if (argument.type !== "AwaitExpression" || argument.argument.type !== "CallExpression" || argument.argument.callee.type !== "MemberExpression" || !isPropertyAccessor(argument.argument.callee, "count")) {
          return;
        }
        const callee = argument.argument.callee;
        context.report({
          fix(fixer) {
            return [
              // remove the "await" expression
              fixer.removeRange([
                argument.range[0],
                argument.range[0] + "await".length + 1
              ]),
              // remove the "count()" method accessor
              fixer.removeRange([
                callee.property.range[0] - 1,
                argument.argument.range[1]
              ]),
              // replace the current matcher with "toHaveCount"
              replaceAccessorFixer(fixer, expectCall.matcher, "toHaveCount"),
              // insert "await" to before "expect()"
              fixer.insertTextBefore(node, "await ")
            ];
          },
          messageId: "useToHaveCount",
          node: expectCall.matcher
        });
      }
    };
  },
  meta: {
    docs: {
      category: "Best Practices",
      description: "Suggest using `toHaveCount()`",
      recommended: false,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/prefer-to-have-count.md"
    },
    fixable: "code",
    messages: {
      useToHaveCount: "Use toHaveCount() instead"
    },
    schema: [],
    type: "suggestion"
  }
};

// src/rules/prefer-to-have-length.ts
var prefer_to_have_length_default = {
  create(context) {
    return {
      CallExpression(node) {
        const expectCall = parseExpectCall(context, node);
        if (!expectCall || !equalityMatchers.has(expectCall.matcherName)) {
          return;
        }
        const [argument] = node.arguments;
        if (argument?.type !== "MemberExpression" || !isPropertyAccessor(argument, "length")) {
          return;
        }
        context.report({
          fix(fixer) {
            return [
              // remove the "length" property accessor
              fixer.removeRange([
                argument.property.range[0] - 1,
                argument.range[1]
              ]),
              // replace the current matcher with "toHaveLength"
              replaceAccessorFixer(fixer, expectCall.matcher, "toHaveLength")
            ];
          },
          messageId: "useToHaveLength",
          node: expectCall.matcher
        });
      }
    };
  },
  meta: {
    docs: {
      category: "Best Practices",
      description: "Suggest using `toHaveLength()`",
      recommended: false,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/prefer-to-have-length.md"
    },
    fixable: "code",
    messages: {
      useToHaveLength: "Use toHaveLength() instead"
    },
    schema: [],
    type: "suggestion"
  }
};

// src/rules/prefer-web-first-assertions.ts
var methods3 = {
  getAttribute: {
    matcher: "toHaveAttribute",
    type: "string"
  },
  innerText: { matcher: "toHaveText", type: "string" },
  inputValue: { matcher: "toHaveValue", type: "string" },
  isChecked: {
    matcher: "toBeChecked",
    prop: "checked",
    type: "boolean"
  },
  isDisabled: {
    inverse: "toBeEnabled",
    matcher: "toBeDisabled",
    type: "boolean"
  },
  isEditable: { matcher: "toBeEditable", type: "boolean" },
  isEnabled: {
    inverse: "toBeDisabled",
    matcher: "toBeEnabled",
    type: "boolean"
  },
  isHidden: {
    inverse: "toBeVisible",
    matcher: "toBeHidden",
    type: "boolean"
  },
  isVisible: {
    inverse: "toBeHidden",
    matcher: "toBeVisible",
    type: "boolean"
  },
  textContent: { matcher: "toHaveText", type: "string" }
};
var supportedMatchers = /* @__PURE__ */ new Set([
  "toBe",
  "toEqual",
  "toBeTruthy",
  "toBeFalsy"
]);
function dereference(context, node) {
  if (node.type !== "Identifier") {
    return node;
  }
  const scope = context.sourceCode.getScope(node);
  for (const ref of scope.references) {
    const refParent = ref.identifier.parent;
    if (refParent.type === "VariableDeclarator") {
      return refParent.init;
    }
  }
}
var prefer_web_first_assertions_default = {
  create(context) {
    return {
      CallExpression(node) {
        const expectCall = parseExpectCall(context, node);
        if (!expectCall)
          return;
        const arg = dereference(context, node.arguments[0]);
        if (!arg || arg.type !== "AwaitExpression" || arg.argument.type !== "CallExpression" || arg.argument.callee.type !== "MemberExpression") {
          return;
        }
        if (!supportedMatchers.has(expectCall.matcherName))
          return;
        const method = getStringValue(arg.argument.callee.property);
        const methodConfig = methods3[method];
        if (!methodConfig)
          return;
        const { args, matcher } = expectCall;
        const notModifier = expectCall.modifiers.find(
          (mod) => getStringValue(mod) === "not"
        );
        const isFalsy = methodConfig.type === "boolean" && (!!args.length && isBooleanLiteral(args[0], false) || expectCall.matcherName === "toBeFalsy");
        const isInverse = methodConfig.inverse ? notModifier || isFalsy : notModifier && isFalsy;
        const newMatcher = +!!notModifier ^ +isFalsy && methodConfig.inverse || methodConfig.matcher;
        const { callee } = arg.argument;
        context.report({
          data: {
            matcher: newMatcher,
            method
          },
          fix: (fixer) => {
            const methodArgs = arg.argument.type === "CallExpression" ? arg.argument.arguments : [];
            const methodEnd = methodArgs.length ? methodArgs.at(-1).range[1] + 1 : callee.property.range[1] + 2;
            const fixes = [
              // Add await to the expect call
              fixer.insertTextBefore(node, "await "),
              // Remove the await keyword
              fixer.replaceTextRange(
                [arg.range[0], arg.argument.range[0]],
                ""
              ),
              // Remove the old Playwright method and any arguments
              fixer.replaceTextRange(
                [callee.property.range[0] - 1, methodEnd],
                ""
              )
            ];
            if (isInverse && notModifier) {
              const notRange = notModifier.range;
              fixes.push(fixer.removeRange([notRange[0], notRange[1] + 1]));
            }
            if (!methodConfig.inverse && !notModifier && isFalsy) {
              fixes.push(fixer.insertTextBefore(matcher, "not."));
            }
            fixes.push(fixer.replaceText(matcher, newMatcher));
            const [matcherArg] = args ?? [];
            if (matcherArg && isBooleanLiteral(matcherArg)) {
              fixes.push(fixer.remove(matcherArg));
            } else if (methodConfig.prop && matcherArg) {
              const propArg = methodConfig.prop;
              const variable = getStringValue(matcherArg);
              const args2 = `{ ${propArg}: ${variable} }`;
              fixes.push(fixer.replaceText(matcherArg, args2));
            }
            const hasOtherArgs = !!methodArgs.filter(
              (arg2) => !isBooleanLiteral(arg2)
            ).length;
            if (methodArgs) {
              const range = matcher.range;
              const stringArgs = methodArgs.map((arg2) => getRawValue(arg2)).concat(hasOtherArgs ? "" : []).join(", ");
              fixes.push(
                fixer.insertTextAfterRange(
                  [range[0], range[1] + 1],
                  stringArgs
                )
              );
            }
            return fixes;
          },
          messageId: "useWebFirstAssertion",
          node
        });
      }
    };
  },
  meta: {
    docs: {
      category: "Best Practices",
      description: "Prefer web first assertions",
      recommended: true,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/prefer-web-first-assertions.md"
    },
    fixable: "code",
    messages: {
      useWebFirstAssertion: "Replace {{method}}() with {{matcher}}()."
    },
    type: "suggestion"
  }
};

// src/rules/require-hook.ts
var isNullOrUndefined = (node) => {
  return node.type === "Literal" && node.value === null || isIdentifier(node, "undefined");
};
var shouldBeInHook = (context, node, allowedFunctionCalls = []) => {
  switch (node.type) {
    case "ExpressionStatement":
      return shouldBeInHook(context, node.expression, allowedFunctionCalls);
    case "CallExpression":
      return !(parseFnCall(context, node) || allowedFunctionCalls.includes(getStringValue(node.callee)));
    case "VariableDeclaration": {
      if (node.kind === "const") {
        return false;
      }
      return node.declarations.some(
        ({ init }) => init != null && !isNullOrUndefined(init)
      );
    }
    default:
      return false;
  }
};
var require_hook_default = {
  create(context) {
    const options = {
      allowedFunctionCalls: [],
      ...context.options?.[0] ?? {}
    };
    const checkBlockBody = (body) => {
      for (const statement of body) {
        if (shouldBeInHook(context, statement, options.allowedFunctionCalls)) {
          context.report({
            messageId: "useHook",
            node: statement
          });
        }
      }
    };
    return {
      CallExpression(node) {
        if (!isDescribeCall(node) || node.arguments.length < 2) {
          return;
        }
        const [, testFn] = node.arguments;
        if (!isFunction(testFn) || testFn.body.type !== "BlockStatement") {
          return;
        }
        checkBlockBody(testFn.body.body);
      },
      Program(program) {
        checkBlockBody(program.body);
      }
    };
  },
  meta: {
    docs: {
      category: "Best Practices",
      description: "Require setup and teardown code to be within a hook",
      recommended: false,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/require-hook.md"
    },
    messages: {
      useHook: "This should be done within a hook"
    },
    schema: [
      {
        additionalProperties: false,
        properties: {
          allowedFunctionCalls: {
            items: { type: "string" },
            type: "array"
          }
        },
        type: "object"
      }
    ],
    type: "suggestion"
  }
};

// src/rules/require-soft-assertions.ts
var require_soft_assertions_default = {
  create(context) {
    return {
      CallExpression(node) {
        if (getExpectType(context, node) === "standalone") {
          context.report({
            fix: (fixer) => fixer.insertTextAfter(node.callee, ".soft"),
            messageId: "requireSoft",
            node: node.callee
          });
        }
      }
    };
  },
  meta: {
    docs: {
      description: "Require all assertions to use `expect.soft`",
      recommended: false,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/require-soft-assertions.md"
    },
    fixable: "code",
    messages: {
      requireSoft: "Unexpected non-soft assertion"
    },
    schema: [],
    type: "suggestion"
  }
};

// src/rules/require-top-level-describe.ts
var require_top_level_describe_default = {
  create(context) {
    const { maxTopLevelDescribes } = {
      maxTopLevelDescribes: Infinity,
      ...context.options?.[0] ?? {}
    };
    let topLevelDescribeCount = 0;
    let describeCount = 0;
    return {
      CallExpression(node) {
        if (isDescribeCall(node)) {
          describeCount++;
          if (describeCount === 1) {
            topLevelDescribeCount++;
            if (topLevelDescribeCount > maxTopLevelDescribes) {
              context.report({
                data: getAmountData(maxTopLevelDescribes),
                messageId: "tooManyDescribes",
                node: node.callee
              });
            }
          }
        } else if (!describeCount) {
          if (isTestCall(context, node)) {
            context.report({ messageId: "unexpectedTest", node: node.callee });
          } else if (isTestHook(context, node)) {
            context.report({ messageId: "unexpectedHook", node: node.callee });
          }
        }
      },
      "CallExpression:exit"(node) {
        if (isDescribeCall(node)) {
          describeCount--;
        }
      }
    };
  },
  meta: {
    docs: {
      category: "Best Practices",
      description: "Require test cases and hooks to be inside a `test.describe` block",
      recommended: false,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/require-top-level-describe.md"
    },
    messages: {
      tooManyDescribes: "There should not be more than {{max}} describe{{s}} at the top level",
      unexpectedHook: "All hooks must be wrapped in a describe block.",
      unexpectedTest: "All test cases must be wrapped in a describe block."
    },
    schema: [
      {
        additionalProperties: false,
        properties: {
          maxTopLevelDescribes: {
            minimum: 1,
            type: "number"
          }
        },
        type: "object"
      }
    ],
    type: "suggestion"
  }
};

// src/rules/valid-expect.ts
function isMatcherCalled(node) {
  if (node.parent.type !== "MemberExpression") {
    return {
      called: node.parent.type === "CallExpression" && node.parent.callee === node,
      node
    };
  }
  return isMatcherCalled(node.parent);
}
var valid_expect_default = {
  create(context) {
    const options = {
      maxArgs: 2,
      minArgs: 1,
      ...context.options?.[0] ?? {}
    };
    const minArgs = Math.min(options.minArgs, options.maxArgs);
    const maxArgs = Math.max(options.minArgs, options.maxArgs);
    return {
      CallExpression(node) {
        if (!isExpectCall(context, node))
          return;
        const expectCall = parseExpectCall(context, node);
        if (!expectCall) {
          context.report({ messageId: "matcherNotFound", node });
        } else {
          const result = isMatcherCalled(node);
          if (!result.called) {
            context.report({
              messageId: "matcherNotCalled",
              node: result.node.type === "MemberExpression" ? result.node.property : result.node
            });
          }
        }
        if (node.arguments.length < minArgs) {
          context.report({
            data: getAmountData(minArgs),
            messageId: "notEnoughArgs",
            node
          });
        }
        if (node.arguments.length > maxArgs) {
          context.report({
            data: getAmountData(maxArgs),
            messageId: "tooManyArgs",
            node
          });
        }
      }
    };
  },
  meta: {
    docs: {
      category: "Possible Errors",
      description: "Enforce valid `expect()` usage",
      recommended: true,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/valid-expect.md"
    },
    messages: {
      matcherNotCalled: "Matchers must be called to assert.",
      matcherNotFound: "Expect must have a corresponding matcher call.",
      notEnoughArgs: "Expect requires at least {{amount}} argument{{s}}.",
      tooManyArgs: "Expect takes at most {{amount}} argument{{s}}."
    },
    schema: [
      {
        additionalProperties: false,
        properties: {
          maxArgs: {
            minimum: 1,
            type: "number"
          },
          minArgs: {
            minimum: 1,
            type: "number"
          }
        },
        type: "object"
      }
    ],
    type: "problem"
  }
};

// src/rules/valid-title.ts
var doesBinaryExpressionContainStringNode = (binaryExp) => {
  if (isStringNode(binaryExp.right)) {
    return true;
  }
  if (binaryExp.left.type === "BinaryExpression") {
    return doesBinaryExpressionContainStringNode(binaryExp.left);
  }
  return isStringNode(binaryExp.left);
};
var quoteStringValue = (node) => node.type === "TemplateLiteral" ? `\`${node.quasis[0].value.raw}\`` : node.raw ?? "";
var compileMatcherPattern = (matcherMaybeWithMessage) => {
  const [matcher, message] = Array.isArray(matcherMaybeWithMessage) ? matcherMaybeWithMessage : [matcherMaybeWithMessage];
  return [new RegExp(matcher, "u"), message];
};
var compileMatcherPatterns = (matchers) => {
  if (typeof matchers === "string" || Array.isArray(matchers)) {
    const compiledMatcher = compileMatcherPattern(matchers);
    return {
      describe: compiledMatcher,
      test: compiledMatcher
    };
  }
  return {
    describe: matchers.describe ? compileMatcherPattern(matchers.describe) : null,
    test: matchers.test ? compileMatcherPattern(matchers.test) : null
  };
};
var MatcherAndMessageSchema = {
  additionalItems: false,
  items: { type: "string" },
  maxItems: 2,
  minItems: 1,
  type: "array"
};
var valid_title_default = {
  create(context) {
    const opts = context.options?.[0] ?? {};
    const {
      disallowedWords = [],
      ignoreSpaces = false,
      ignoreTypeOfDescribeName = false,
      ignoreTypeOfTestName = false,
      mustMatch,
      mustNotMatch
    } = opts;
    const disallowedWordsRegexp = new RegExp(
      `\\b(${disallowedWords.join("|")})\\b`,
      "iu"
    );
    const mustNotMatchPatterns = compileMatcherPatterns(mustNotMatch ?? {});
    const mustMatchPatterns = compileMatcherPatterns(mustMatch ?? {});
    return {
      CallExpression(node) {
        const isDescribe = isDescribeCall(node);
        const isTest = isTestCall(context, node);
        if (!isDescribe && !isTest) {
          return;
        }
        const [argument] = node.arguments;
        if (!argument) {
          return;
        }
        if (!isStringNode(argument)) {
          if (argument.type === "BinaryExpression" && doesBinaryExpressionContainStringNode(argument)) {
            return;
          }
          if (!(isDescribe && ignoreTypeOfDescribeName || isTest && ignoreTypeOfTestName) && argument.type !== "TemplateLiteral") {
            context.report({
              loc: argument.loc,
              messageId: "titleMustBeString"
            });
          }
          return;
        }
        const title = getStringValue(argument);
        const functionName = isDescribe ? "describe" : "test";
        if (!title) {
          context.report({
            data: { functionName },
            messageId: "emptyTitle",
            node
          });
          return;
        }
        if (disallowedWords.length > 0) {
          const disallowedMatch = disallowedWordsRegexp.exec(title);
          if (disallowedMatch) {
            context.report({
              data: { word: disallowedMatch[1] },
              messageId: "disallowedWord",
              node: argument
            });
            return;
          }
        }
        if (ignoreSpaces === false && title.trim().length !== title.length) {
          context.report({
            fix: (fixer) => [
              fixer.replaceTextRange(
                argument.range,
                quoteStringValue(argument).replace(/^([`'"]) +?/u, "$1").replace(/ +?([`'"])$/u, "$1")
              )
            ],
            messageId: "accidentalSpace",
            node: argument
          });
        }
        const [firstWord] = title.split(" ");
        if (firstWord.toLowerCase() === functionName) {
          context.report({
            fix: (fixer) => [
              fixer.replaceTextRange(
                argument.range,
                quoteStringValue(argument).replace(/^([`'"]).+? /u, "$1")
              )
            ],
            messageId: "duplicatePrefix",
            node: argument
          });
        }
        const [mustNotMatchPattern, mustNotMatchMessage] = mustNotMatchPatterns[functionName] ?? [];
        if (mustNotMatchPattern && mustNotMatchPattern.test(title)) {
          context.report({
            data: {
              functionName,
              message: mustNotMatchMessage ?? "",
              pattern: String(mustNotMatchPattern)
            },
            messageId: mustNotMatchMessage ? "mustNotMatchCustom" : "mustNotMatch",
            node: argument
          });
          return;
        }
        const [mustMatchPattern, mustMatchMessage] = mustMatchPatterns[functionName] ?? [];
        if (mustMatchPattern && !mustMatchPattern.test(title)) {
          context.report({
            data: {
              functionName,
              message: mustMatchMessage ?? "",
              pattern: String(mustMatchPattern)
            },
            messageId: mustMatchMessage ? "mustMatchCustom" : "mustMatch",
            node: argument
          });
          return;
        }
      }
    };
  },
  meta: {
    docs: {
      category: "Best Practices",
      description: "Enforce valid titles",
      recommended: true,
      url: "https://github.com/playwright-community/eslint-plugin-playwright/tree/main/docs/rules/valid-title.md"
    },
    fixable: "code",
    messages: {
      accidentalSpace: "should not have leading or trailing spaces",
      disallowedWord: '"{{ word }}" is not allowed in test titles',
      duplicatePrefix: "should not have duplicate prefix",
      emptyTitle: "{{ functionName }} should not have an empty title",
      mustMatch: "{{ functionName }} should match {{ pattern }}",
      mustMatchCustom: "{{ message }}",
      mustNotMatch: "{{ functionName }} should not match {{ pattern }}",
      mustNotMatchCustom: "{{ message }}",
      titleMustBeString: "Title must be a string"
    },
    schema: [
      {
        additionalProperties: false,
        patternProperties: {
          [/^must(?:Not)?Match$/u.source]: {
            oneOf: [
              { type: "string" },
              MatcherAndMessageSchema,
              {
                additionalProperties: {
                  oneOf: [{ type: "string" }, MatcherAndMessageSchema]
                },
                propertyNames: { enum: ["describe", "test"] },
                type: "object"
              }
            ]
          }
        },
        properties: {
          disallowedWords: {
            items: { type: "string" },
            type: "array"
          },
          ignoreSpaces: {
            default: false,
            type: "boolean"
          },
          ignoreTypeOfDescribeName: {
            default: false,
            type: "boolean"
          },
          ignoreTypeOfTestName: {
            default: false,
            type: "boolean"
          }
        },
        type: "object"
      }
    ],
    type: "suggestion"
  }
};

// src/index.ts
var index = {
  configs: {},
  rules: {
    "expect-expect": expect_expect_default,
    "max-expects": max_expects_default,
    "max-nested-describe": max_nested_describe_default,
    "missing-playwright-await": missing_playwright_await_default,
    "no-commented-out-tests": no_commented_out_tests_default,
    "no-conditional-expect": no_conditional_expect_default,
    "no-conditional-in-test": no_conditional_in_test_default,
    "no-duplicate-hooks": no_duplicate_hooks_default,
    "no-element-handle": no_element_handle_default,
    "no-eval": no_eval_default,
    "no-focused-test": no_focused_test_default,
    "no-force-option": no_force_option_default,
    "no-get-by-title": no_get_by_title_default,
    "no-hooks": no_hooks_default,
    "no-nested-step": no_nested_step_default,
    "no-networkidle": no_networkidle_default,
    "no-nth-methods": no_nth_methods_default,
    "no-page-pause": no_page_pause_default,
    "no-raw-locators": no_raw_locators_default,
    "no-restricted-matchers": no_restricted_matchers_default,
    "no-skipped-test": no_skipped_test_default,
    "no-standalone-expect": no_standalone_expect_default,
    "no-unsafe-references": no_unsafe_references_default,
    "no-useless-await": no_useless_await_default,
    "no-useless-not": no_useless_not_default,
    "no-wait-for-selector": no_wait_for_selector_default,
    "no-wait-for-timeout": no_wait_for_timeout_default,
    "prefer-comparison-matcher": prefer_comparison_matcher_default,
    "prefer-equality-matcher": prefer_equality_matcher_default,
    "prefer-hooks-in-order": prefer_hooks_in_order_default,
    "prefer-hooks-on-top": prefer_hooks_on_top_default,
    "prefer-lowercase-title": prefer_lowercase_title_default,
    "prefer-strict-equal": prefer_strict_equal_default,
    "prefer-to-be": prefer_to_be_default,
    "prefer-to-contain": prefer_to_contain_default,
    "prefer-to-have-count": prefer_to_have_count_default,
    "prefer-to-have-length": prefer_to_have_length_default,
    "prefer-web-first-assertions": prefer_web_first_assertions_default,
    "require-hook": require_hook_default,
    "require-soft-assertions": require_soft_assertions_default,
    "require-top-level-describe": require_top_level_describe_default,
    "valid-expect": valid_expect_default,
    "valid-title": valid_title_default
  }
};
var sharedConfig = {
  rules: {
    "no-empty-pattern": "off",
    "playwright/expect-expect": "warn",
    "playwright/max-nested-describe": "warn",
    "playwright/missing-playwright-await": "error",
    "playwright/no-conditional-expect": "warn",
    "playwright/no-conditional-in-test": "warn",
    "playwright/no-element-handle": "warn",
    "playwright/no-eval": "warn",
    "playwright/no-focused-test": "error",
    "playwright/no-force-option": "warn",
    "playwright/no-nested-step": "warn",
    "playwright/no-networkidle": "error",
    "playwright/no-page-pause": "warn",
    "playwright/no-skipped-test": "warn",
    "playwright/no-standalone-expect": "error",
    "playwright/no-unsafe-references": "error",
    "playwright/no-useless-await": "warn",
    "playwright/no-useless-not": "warn",
    "playwright/no-wait-for-selector": "warn",
    "playwright/no-wait-for-timeout": "warn",
    "playwright/prefer-web-first-assertions": "error",
    "playwright/valid-expect": "error",
    "playwright/valid-title": "error"
  }
};
var legacyConfig = {
  ...sharedConfig,
  env: {
    "shared-node-browser": true
  },
  plugins: ["playwright"]
};
var flatConfig = {
  ...sharedConfig,
  languageOptions: {
    globals: import_globals.default["shared-node-browser"]
  },
  plugins: {
    playwright: index
  }
};
var sharedJestConfig = {
  rules: {
    "jest/no-standalone-expect": [
      "error",
      {
        additionalTestBlockFunctions: [
          "test.jestPlaywrightDebug",
          "it.jestPlaywrightDebug",
          "test.jestPlaywrightSkip",
          "it.jestPlaywrightSkip",
          "test.jestPlaywrightConfig",
          "it.jestPlaywrightConfig"
        ]
      }
    ],
    "playwright/missing-playwright-await": "error",
    "playwright/no-page-pause": "warn"
  }
};
var legacyJestConfig = {
  ...sharedJestConfig,
  env: {
    jest: true,
    "shared-node-browser": true
  },
  globals: {
    browser: true,
    browserName: true,
    context: true,
    deviceName: true,
    jestPlaywright: true,
    page: true
  },
  plugins: ["jest", "playwright"]
};
var jestConfig = {
  ...sharedJestConfig,
  languageOptions: {
    globals: {
      ...import_globals.default["shared-node-browser"],
      ...import_globals.default.jest,
      browser: "writable",
      browserName: "writable",
      context: "writable",
      deviceName: "writable",
      jestPlaywright: "writable",
      page: "writable"
    }
  },
  plugins: {
    playwright: index
  }
};
module.exports = {
  ...index,
  configs: {
    "flat/jest-playwright": jestConfig,
    "flat/recommended": flatConfig,
    "jest-playwright": legacyJestConfig,
    "playwright-test": legacyConfig,
    recommended: legacyConfig
  }
};
